/*
    Sega Saturn Timer 0/1 example program
    by Charles MacDonald
    WWW: http://cgfm2.emuviews.com

    Based on the bmp8bpp.zip and startup.zip packages written by
    Bart Trzynadlowski. (www.dynarec.com/~bart)

    Uses the SH-COFF toolchain from KPIT Infosystems. (www.kpit.com)
    Load and run at address $06004000.

    This program uses timer 0 and timer 1 to split the screen horizontally
    and vertically, at which point the screen resolution is changed. This
    allows a single frame to simulataneously show high and low resolution
    graphics. The H-Blank IN interrupt sets the display mode to hi-res and
    then adjusts the interrupt level in SR and sleeps to timer 1 can start
    up afterwards. The timer 1 interrupt sets the display mode to lo-res.
    The timer 0 interrupt turns off timer 1 and the H-Blank IN interrupt,
    both of which are re-enabled in the V-Blank OUT interrupt.

    Due to timing issues the right side of the display will never be
    completely flicker free. I found that using the SLEEP instruction was the
    best way to reduce flicker, otherwise the main loop code (where it waits
    for VBlank) throws the timer 1 response off and causes a lot of distortion.

    The horizontal scrolling could be modified on each line and cleared in
    the H-Blank IN interrupt to stop the right side of the display from
    moving around when the horizontal split point is changed, which is a side
    effect from the high resolution portion of the display advancing the
    VDP2's offset within the line too far.

    Controls:

    Left/Right - Move horizontal split point
    Left/Right shoulder buttons - Move horizontal split point by one
    Up/Down - Move vertical split point
    Start - Return to cheat cartridge program
*/

#include "shared.h"

int vcount = 112;
int hcount = 0x00FF;

void _main()
{
    int update = 1;
    int x, y;
    uint8 *bmp = (uint8 *)VDP2_VRAM;

    vdp_init();
    conio_init();
    pad_init();
    irq_init();

    /* Draw test image */
    for(y=0;y<224;y++)
    for(x=0;x<1024;x++)
        bmp[y*1024+x] = 0x10+(((x*x-y*y)>>3)&0x0F);

    /* Print messages and cursor */
    _printf(2, 2, 0xF0, "Timer 0/1 Example");
    _printf(2, 3, 0xF0, "V Pos. = ");
    _printf(2, 4, 0xF0, "H Pos. = ");

    _printf(0, 10, 0xF0, "00000000000000001111111111111111222222222222222233333333333333334444444444444444555555555555555566666666666666667777777777777777");
    _printf(0, 11, 0xF0, "0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF");

    /* Set up timer and interrupt registers */
    SCU_TIMER0_COMPARE = vcount;
    SCU_TIMER1_SET     = hcount;
    SCU_TIMER1_CONTROL = 0x0101;
    SCU_IRQ_MASK = irq_mask = (0xBFFF & ~(SCU_IRQ_TIMER0 | SCU_IRQ_TIMER1 | SCU_IRQ_VBLANK_OUT | SCU_IRQ_HBLANK_IN | SCU_IRQ_VBLANK_IN));

    /* Enable interrupts */
    set_sr(old_sr & 0xFFFFFF0F);

    /* Screen on */
    TVMD = 0x8000;

    for(;;)
    {
        /* Wait fo VBlank */
        wait_vblank_out();
        wait_vblank_in();

        /* Poll joypad */
        pad_read(0);

        /* Start - Quit program */
        if(pad[0].new & PAD_START)
            break;

        /* Adjust vertical & horizontal split point */
        if((pad[0].new & PAD_LEFT) || (pad[0].delta & PAD_LBUTTON)) {
            hcount--;
            if(hcount < 0) hcount = 0;
            update = 1;
        }

        if((pad[0].new & PAD_RIGHT) || (pad[0].delta & PAD_RBUTTON)) {
            hcount++;
            if(hcount > 0x1FF) hcount = 0x1FF;
            update = 1;
        }

        if(pad[0].new & PAD_UP) {
            vcount--;
            if(vcount < 0) vcount = 0;
            update = 1;
        }

        if(pad[0].new & PAD_DOWN) {
            vcount++;
            if(vcount > 0x3FF) vcount = 0x3FF;
            update = 1;
        }

        if(update)
        {
            update = 0;
            SCU_TIMER1_SET = hcount;
            SCU_TIMER0_COMPARE = vcount;
            _printhexl(11, 4, 0xF0, hcount);
            _printhexl(11, 3, 0xF0, vcount);
        }
    }

    /* Mask interrupts */
    set_sr(old_sr | 0x000000F0);

    irq_shutdown();
    vdp_shutdown();
    pad_shutdown();
}

